#include "font.h"
#include <fstream>
using namespace std;

Font::Font(Texture tex, int tw, int th, int size, const char* infofile): tex(tex), size(size), tw(tw), th(th)
{
	ifstream in(infofile);

	memset(chars,0,256*sizeof(charinfo));

	while (!in.eof()) {
		int line[7];
		for (int i=0; i<7; i++) in >> line[i];

		if (line[1] != size) continue;

		charinfo ci;
		ci.baseline = line[2];
		ci.x = line[3];
		ci.y = line[4];
		ci.w = line[5];
		ci.h = line[6];

		ci.tx1 = ci.x / (float)tw;
		ci.tx2 = (ci.x + ci.w) / (float)tw;
		ci.ty1 = ci.y / (float)th;
		ci.ty2 = (ci.y + ci.h) / (float)th;

		chars[line[0]] = ci;
	}
	charinfo space;
	space.baseline = 0;
	space.h = size;
	space.w = size / 2;
	space.x = space.y = 0;
	space.tx1 = space.tx2 = space.ty1 = space.ty2 = 0;
	chars[32] = space;

}

Font::~Font()
{
}

void Font::drawchar(int x, int y, const char ch)
{
	charinfo *c = &chars[ch];

    glBegin(GL_QUADS);

	glTexCoord2f(c->tx1,c->ty1);
	glVertex2f((float)x, (float)y);

	glTexCoord2f(c->tx2,c->ty1);
	glVertex2f((float)(x+c->w), (float)y);

	glTexCoord2f(c->tx2,c->ty2);
	glVertex2f((float)(x+c->w), (float)(y+c->h));

	glTexCoord2f(c->tx1,c->ty2);
	glVertex2f((float)x, (float)(y+c->h));

	glEnd();

}

// ASSUME: 2D mode
void Font::drawtext(int x, int y, const char *text)
{
	glBindTexture(GL_TEXTURE_2D, tex);

    int base = y + size;
	int xpos = x;
    const char *c = text;
	while (*c!=0) {

		if (*c != '\n' && *c != '\r') {
			drawchar(xpos, base - chars[*c].baseline, *c);
			xpos += chars[*c].w;
		} else {
			base += size;
			xpos = x;
		}
		c++;
	}
}


