#include "game.h"
#include "burglar.h"

#include <string>
using namespace std;

Texture texBusted, texVictory;

Menu *gMenu;

Game::Game(const char *filename, const char *music)
{
    w.init(filename);
	fw = turn = st = 0;
	w.gameStart();
	texBusted = video.loadTexture("busted.png");
	texVictory = video.loadTexture("victory.png");

	playMusic(music);
}

Game::~Game()
{
	shutup();
}

void Game::key(SDL_Event event)
{
	SDLKey ks = event.key.keysym.sym;

	if (event.type == SDL_KEYDOWN) {
		if (ks == SDLK_UP || ks==SDLK_w) fw = 1.0f;
		if (ks == SDLK_DOWN || ks==SDLK_s) fw = -1.0f;
		if (ks == SDLK_LEFT || ks==SDLK_q) turn = -1.0f;
		if (ks == SDLK_RIGHT || ks==SDLK_e) turn = 1.0f;
		if (ks == SDLK_a || ks==SDLK_COMMA) st = -1.0f;
		if (ks == SDLK_d || ks==SDLK_PERIOD) st = 1.0f;
	}

	if (event.type == SDL_KEYUP) {
		if (ks == SDLK_UP || ks==SDLK_w) if (fw>0) fw = 0;
		if (ks == SDLK_DOWN || ks==SDLK_s) if (fw<0) fw = 0;
		if (ks == SDLK_LEFT || ks==SDLK_q) if (turn<0) turn = 0;
		if (ks == SDLK_RIGHT || ks==SDLK_e) if (turn>0) turn = 0;
		if (ks == SDLK_d || ks==SDLK_PERIOD) if (st>0) st = 0;
		if (ks == SDLK_a || ks==SDLK_COMMA) if (st<0) st = 0;

		if (ks == SDLK_SPACE || ks == SDLK_RETURN) {
			if (w.gameover) w.gameStart();
			else if (w.victory) {
				delete gMenu->g;
				gMenu->g = 0;
			}
		}
	}

}



void Game::update(float dt)
{
	w.player.fw = fw;
	w.player.turn = turn;
	w.player.strafe = st;

	while (dt>0.1f) {
		w.update(0.1f);
		dt -= 0.1f;
	}
	w.update(dt);

}

void Game::draw()
{
	video.set3D();
	w.draw(w.player.pos, w.player.dir+180.0f);

	glDisable(GL_LIGHTING);
	video.set2D();
	// draw ui
	glColor4f(1,0,0,0.5f);
	glBegin(GL_QUADS);
	float y = video.yres * (1.0f - w.awareness);
	glVertex2f(0,y);
	glVertex2f(50,y);
	glVertex2i(50,video.yres);
	glVertex2i(0,video.yres);
	glEnd();
	glColor4f(1,1,1,1);

	glEnable(GL_TEXTURE_2D);

	if (w.victory || w.gameover) {

		if (w.gameover) {
			glBindTexture(GL_TEXTURE_2D, texBusted);
		}
		else if (w.victory) {
			glBindTexture(GL_TEXTURE_2D, texVictory);
		}

		glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex2i(video.xres/2 - 384, video.yres/2 - 384);

		glTexCoord2f(1,0);
		glVertex2i(video.xres/2 + 384, video.yres/2 - 384);

		glTexCoord2f(1,1);
		glVertex2i(video.xres/2 + 384, video.yres/2 + 384);

		glTexCoord2f(0,1);
		glVertex2i(video.xres/2 - 384, video.yres/2 + 384);

		glEnd();
	}

	char buf[32];
	sprintf(buf,"%d/%d",w.collected, w.total);
	f16->drawtext(0,0,buf);

}


Menu::Menu()
{
	g = 0;
	sel = 0;
	//g = new Game("map.txt");
	gMenu = this;
}

string labels[] = {"1. Museum", "2. Cathedral", "3. Underground base", "Quit"};
string filenames[] = {"data/map.txt", "data/map2.txt", "data/map3.txt", "quit"};
string music[] = {"data/burglar.it", "data/cathedral.it", "data/base.it"};
const int num = 4;


void Menu::draw()
{
	if (g) {
		g->draw();
		return;
	}

	// menu
	video.set2D();

	glEnable(GL_TEXTURE_2D);

	
	f32->drawtext(400,50, "- Burglar -");
	
	// draw title scrn
	int y = 300;
	for (int i=0; i<num; i++) {

		if (i==sel) glColor4f(1,0.3f,0.3f,1);
		else glColor4f(1,1,1,1);

		f24->drawtext(400, y, labels[i].c_str());
		y += 32;

	}

	glColor4f(1,1,1,1);
	f16->drawtext(20,670,
		"Move, turn: arrows\n"
		"Strafe: A, D, dot, comma\n"
		"Objective: get all the treasure and escape.\n"
		"Avoid the light."
	);


	video.set3D();

	glLoadIdentity();

	glEnable(GL_LIGHTING);

	GLfloat ld[4] = {1,1,1,1};
	GLfloat la[4] = {0,0,0,1};
	GLfloat lp[4] = {0.1f,-0.1f,1,0};
	glLightfv(GL_LIGHT0, GL_AMBIENT, la);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, ld);
	glLightfv(GL_LIGHT0, GL_POSITION, lp);
	glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 180.0f);
	glEnable(GL_LIGHT0);

	for (int i=GL_LIGHT1; i<=GL_LIGHT7; i++) glDisable(i);

	
	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, ninja->texture);
	glTranslatef(-2,0,-5);

	const float s = 1.0f;
	glScalef(s,s,s);
    ninja->draw();
	glPopMatrix();

	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, cop->texture);
	glTranslatef(2,0,-5);

	glScalef(s,s,s);
    cop->draw();
	glPopMatrix();


}

void Menu::update(float dt)
{
	SDL_Event event;
	while ( SDL_PollEvent(&event) ) {
		if ( event.type == SDL_QUIT ) {
			done = 1;
		}
		else if ((event.type == SDL_KEYDOWN) && event.key.keysym.sym == SDLK_ESCAPE) {
			//done = 1;
			if (g) {
				delete g;
				g = 0;
			} else done = 1;
		}
		else {
			if (g) g->key(event);
			else if (event.type == SDL_KEYDOWN) {
				// menu
				if (event.key.keysym.sym == SDLK_RETURN) {
					if (filenames[sel]=="quit") {
						done = 1;
					}
					else {
						g = new Game(filenames[sel].c_str(), music[sel].c_str());
					}
				}
				if (event.key.keysym.sym == SDLK_DOWN) {
					sel++;
					if (sel==num) sel=0;
				}
				if (event.key.keysym.sym == SDLK_UP) {
					sel--;
					if (sel==-1) sel=num-1;
				}


			}
		}
	}

	if (g) {
		g->update(dt);
		return;
	}

}

