
#include "model.h"
#include <iostream>
#include <fstream>
#include <cmath>

#include <gl/glut.h>

Model::Model(char *name, char *tex, float scaleFactor)
{
	// model format generated by my crappy python exporter for gamespace :D

	std::ifstream in(name);
	int numUV, i;
	int numFrames;
	
	// read in numbers of things
	in >> numVertices >> numFaces >> numUV >> numFrames;
	in >> sx >> sz >> sy;
	
	// init stuff
	int *uvtemp1 = new int[numFaces*3];
	int *uvtemp2 = uvtemp1 + numFaces;
	int *uvtemp3 = uvtemp2 + numFaces;
	
	faces = new ModelFace[numFaces];
	vertices = new float[numVertices*3];
	normals = new float[numVertices*3];


	// read face data
	for (i=0; i<numFaces; i++) {
		in >> faces[i].vert1 >> faces[i].vert2 >> faces[i].vert3;
		in >> uvtemp1[i] >> uvtemp2[i] >> uvtemp3[i];
		faces[i].vert1 *= 3;
		faces[i].vert2 *= 3;
		faces[i].vert3 *= 3;
	}

	float *ucoords = new float[numUV], *vcoords = new float[numUV];

	// read UV data
	for (i=0; i<numUV; i++) {
		in >> ucoords[i] >> vcoords[i];
		vcoords[i] = 1.0f - vcoords[i];
	}

	// copy over UV data
	for (i=0; i<numFaces; i++) {
		faces[i].u1 = ucoords[uvtemp1[i]];
		faces[i].v1 = vcoords[uvtemp1[i]];
		faces[i].u2 = ucoords[uvtemp2[i]];
		faces[i].v2 = vcoords[uvtemp2[i]];
		faces[i].u3 = ucoords[uvtemp3[i]];
		faces[i].v3 = vcoords[uvtemp3[i]];
	}

	delete[] uvtemp1;
	delete[] ucoords;
	delete[] vcoords;

	// read vertex data
	for (int j=0; j<numVertices*3; ) {
		float x, y, z;
		// swap Z and Y
		in >> x >> z >> y;
		vertices[j++] = x * scaleFactor;
		vertices[j++] = y * scaleFactor;
		vertices[j++] = z * scaleFactor;
	}

	in.close();

	memset(normals, 0, sizeof(float)*numVertices*3);
	
	// calculate normals
	float *v = vertices;
	float *n = normals;
	// face normals
	for (int j=0; j<numFaces; j++) {
		ModelFace &f = faces[j];
		float x1,y1,z1,x2,y2,z2, nx, ny, nz;
		x1 = v[f.vert2] - v[f.vert1];
		x2 = v[f.vert3] - v[f.vert1];
		y1 = v[f.vert2+1] - v[f.vert1+1];
		y2 = v[f.vert3+1] - v[f.vert1+1];
		z1 = v[f.vert2+2] - v[f.vert1+2];
		z2 = v[f.vert3+2] - v[f.vert1+2];
		// cross product
		nx = y1*z2-z1*y2;
		ny = z1*x2-x1*z2;
		nz = x1*y2-y1*x2;
		// normalize face normal
		// is this step required?? :|
		/*float rlen = 1.0f/sqrtf(nx*nx + ny*ny + nz*nz);
		nx *= rlen;
		ny *= rlen;
		nz *= rlen;*/

		n[f.vert1] += nx;
		n[f.vert1+1] += ny;
		n[f.vert1+2] += nz;
		n[f.vert2] += nx;
		n[f.vert2+1] += ny;
		n[f.vert2+2] += nz;
		n[f.vert3] += nx;
		n[f.vert3+1] += ny;
		n[f.vert3+2] += nz;

	}
	// normalize vertex normals
	for (int j=0; j<numVertices*3; j+=3) {
		float nx = n[j], ny = n[j+1], nz = n[j+2];
		float rlen = 1.0f/sqrtf(nx*nx+ny*ny+nz*nz);
		n[j] *= rlen;
		n[j+1] *= rlen;
		n[j+2] *= rlen;
	}

	if (tex && strlen(tex)) {
		texture = video.loadTexture(tex);
	} else texture = 0;

	getList();

}

Model::~Model()
{
	delete[] vertices;
	delete[] normals;
	delete[] faces;
}

void Model::display()
{
	glBegin(GL_TRIANGLES);
	for (int i=0; i<numFaces; i++) {
		ModelFace &f = faces[i];
		glTexCoord2f(f.u1, f.v1);
		glNormal3f(normals[f.vert1], normals[f.vert1+1], normals[f.vert1+2]);
		glVertex3f(vertices[f.vert1], vertices[f.vert1+1], vertices[f.vert1+2]);
		glTexCoord2f(f.u2, f.v2);
		glNormal3f(normals[f.vert2], normals[f.vert2+1], normals[f.vert2+2]);
		glVertex3f(vertices[f.vert2], vertices[f.vert2+1], vertices[f.vert2+2]);
		glTexCoord2f(f.u3, f.v3);
		glNormal3f(normals[f.vert3], normals[f.vert3+1], normals[f.vert3+2]);
		glVertex3f(vertices[f.vert3], vertices[f.vert3+1], vertices[f.vert3+2]);
	}
	glEnd();
}


int Model::getList()
{
	dlist = glGenLists(1);

	glNewList(dlist, GL_COMPILE);

	this->display();

	glEndList();
	return dlist;
}

void Model::draw()
{
    glCallList(dlist);
}
