#include "video.h"

Video video;


void Video::init(int xres, int yres, bool fs)
{
	SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO);
	int flags = SDL_OPENGL | SDL_HWSURFACE | SDL_ANYFORMAT | SDL_DOUBLEBUF;
	if (fs) flags |= SDL_FULLSCREEN;
	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 32);
	primary = SDL_SetVideoMode(xres, yres, 32, flags);
	glViewport(0, 0, xres, yres);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,(GLfloat)xres/(GLfloat)yres, 0.1f, 256.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glColor4f (1.0, 1.0, 1.0, 1.0);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_CULL_FACE);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glAlphaFunc (GL_GREATER, 0);

	this->xres = xres;
	this->yres = yres;
}

void Video::close()
{
	SDL_Quit();
}


Texture Video::loadTexture(std::string filename)
{
	if (textureCache.find(filename) != textureCache.end()) {
		return textureCache[filename];
	}

	std::string fn = "data/" + filename;

	SDL_Surface *tex = IMG_Load(fn.c_str());
    if (tex==0) return 0;

	GLuint format;
	int Bpp = tex->format->BytesPerPixel, bppformat;
	if (Bpp == 3) {
		format = GL_RGB;
		bppformat = GL_RGB8;
	} else if (Bpp == 4) {
		format = GL_RGBA;
		bppformat = GL_RGBA8;
	} else {
		format = GL_RGB;
		bppformat = GL_RGB8;
	}

	GLuint t;

	glGenTextures(1,&t);
	glBindTexture(GL_TEXTURE_2D, t);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, tex->pitch / tex->format->BytesPerPixel);

	//glTexImage2D(GL_TEXTURE_2D, 0, bppformat,
	//	s->w, s->h, 0, format, GL_UNSIGNED_BYTE, s->pixels);

	gluBuild2DMipmaps(GL_TEXTURE_2D, format, tex->w, tex->h, format, GL_UNSIGNED_BYTE, tex->pixels);

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	//glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

	SDL_FreeSurface(tex);

	textureCache[filename] = t;

	return t;
}

void Video::unloadTexture(Texture tex)
{
	glDeleteTextures(1, &tex);
	for (std::map<std::string,Texture>::iterator it = textureCache.begin(); it != textureCache.end(); ++it) {
		if (it->second == tex) {
			textureCache.erase(it);
			break;
		}
	}
}


void Video::set3D()
{
	// prespective projection ("3D mode")

	//glViewport(0, 0, xres, yres);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,(GLfloat)xres/(GLfloat)yres, 0.1f, 256.0f);
	glMatrixMode(GL_MODELVIEW);
	
	glColor4f (1.0, 1.0, 1.0, 1.0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glDisable(GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glDisable(GL_ALPHA_TEST);
	glAlphaFunc (GL_GREATER, 0);
	
}

void Video::set2D()
{
	// ortho projection ("2D mode")
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);

	//glViewport(0, 0, xres, yres);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, xres, yres, 0, -1.0, 1.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glDisable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_ALPHA_TEST);
	glAlphaFunc (GL_GREATER, 0);
}

