#include "world.h"

#include "burglar.h"
#include "frustum.h"

#include <cstdio>
#include <string>
#include <map>
using namespace std;


const float cam_dist = 6.0f;
const float cam_height = 5.0f;

const float WALL_HEIGHT = 1.5f;

const float hr2 = 0.70710678f;


struct TexPair {
	std::string floorTex, wallTex;
};

int holetex;

bool badchar(char c1)
{
	return c1=='.' || c1=='/' || c1=='\\' || (c1>='0' && c1<='9') || (c1>='A' && c1<='Z') || c1=='@' || c1=='$';
}

void World::init(const char *filename)
{
	std::vector<std::string> guardPatterns;
	char buf[1024];

	holetex = video.loadTexture("hole.png");

    FILE *f = fopen(filename, "r");
	int w=0, h=0;
	while (1) {
		fgets(buf, 1024, f);
		if (buf[0]==0 || buf[0]==13 || buf[0]==10) break;
        if (h++==0) w = (int)strlen(buf);
		smap.push_back(string(buf));
	}

	width = w / 2;
	height = h / 2;

	tiles = new Tile[width*height];

	std::map<char, TexPair> texMap;

	while (1) {
		fgets(buf, 1024, f);
		if (buf[0]==0 || buf[0]==13 || buf[0]==10) break;
		char c = buf[0], t1[256], t2[256];
		
		sscanf((char*)buf + 2, "%s %s\n", t1, t2);
		TexPair tp;
		tp.floorTex = string(t1);
		tp.wallTex = string(t2);
        texMap[c] = tp;
	}

	while (1) {
		fgets(buf, 1024, f);
		if (buf[0]==0 || buf[0]==13 || buf[0]==10 || feof(f)) break;
		guardPatterns.push_back(string(buf));
	}

	lid = 0;
	total = 0;

	for (int j=1; j<h-1; j+=2) {
		// tiles
		for (int i=1; i<w-1; i+=2) {
			// actual tile
			char c = smap[j][i];

			int ix = (i-1)/2;
			int iy = (j-1)/2;

			Vec2D fpos((float)ix, (float)iy);
			Vec2D mpos = fpos + Vec2D(0.5f,0.5f);

			if ((c>='0' && c<='9') || (c>='A' && c<='Z')) {
				// guard? bleh

				int guardnum;
				if (c>='0' && c<='9') guardnum = c - '0';
				else guardnum = 10 + (c - 'A');

				// create a guard
				Guard *g = new Guard();
				g->pos = g->pos0 = mpos;
				g->dir = 0;
				switch (guardPatterns[guardnum][1]) {
				case '<': g->dir = 180; break;
				case '>': g->dir = 0; break;
				case '^': g->dir = 270; break;
				case 'v': g->dir = 90; break;
				}
				g->dir0 = g->dir;
				g->pattern = guardPatterns[guardnum].substr(2);
				g->w = this;
				// add a new light, etc
				Light l;
				l.pos = mpos;
				l.dir = 0;
				l.conesize = 30.0f;
				l.dist = 4.0f;
				l.w = this;
				l.color = Vec3D(1,1,1);

				switch(guardPatterns[guardnum][0]) {
				case 'S': // static
					l.color=Vec3D(1,0.1f,0.1f);
					l.dist = 5.0f;
					break;
				case 'P': // patrol?
					l.color=Vec3D(0.1f,1,0.1f);
					l.conesize *= 1.6f;
					g->spd = 1.5f;
					break;
				}

				lights[lid] = l;
				g->light = lid++;

				objects.push_back(g);
			}

			Tile *t = getTile(ix,iy);

			if (c=='@') {
                player.pos = mpos;
				pstart = mpos;
				player.dir = 0;
				player.w = this;
				t->hole = true;
			}

			if (c=='$') {
				Treasure treasure;
                treasure.pos = mpos;
				tstart = mpos;
				treasure.w = this;
				total++;

				treasures.push_back(treasure);
			}

			if (c!='.') {

				// add the tile
				t->w = this;
				t->visible = true;
				t->pos = fpos;
				t->mpos = mpos;

				t->c1 = Vec3D(fpos.x, 0, fpos.y);
				t->c2 = Vec3D(fpos.x+1.0f, WALL_HEIGHT, fpos.y+1.0f);

				char c1=c;
				if (badchar(c1)) {
					c1 = smap[j][i-2];
					if (badchar(c1)) c1 = smap[j][i+2];
					if (badchar(c1)) c1 = smap[j-2][i];
					if (badchar(c1)) c1 = smap[j+2][i];
					if (badchar(c1)) c1 = '#'; // bleh whatever default
				}

				t->tex = video.loadTexture(texMap[c1].floorTex);
				Wall w;
				w.tex = video.loadTexture(texMap[c1].wallTex);

				switch (c) {
				case '/':
					if (smap[j-2][i]=='.') {
						t->type = TILE_SE;
						w.p2 = fpos + Vec2D(1,0);
						w.p1 = fpos + Vec2D(0,1);
						w.normal = Vec3D(hr2,0,hr2);
					}
					else {
						t->type = TILE_NW;
						w.p1 = fpos + Vec2D(1,0);
						w.p2 = fpos + Vec2D(0,1);
						w.normal = Vec3D(-hr2,0,-hr2);
					}
					// add wall
					w.type = WALL_WALL;
					w.sides = WALL_SIDE_FRONT;
					walls.push_back(w);
					t->walls[t->nwalls++] = walls.size()-1;
					break;
				case '\\':
					if (smap[j-2][i]=='.') {
						t->type = TILE_SW;
						w.p1 = fpos;
						w.p2 = fpos + Vec2D(1,1);
						w.normal = Vec3D(-hr2,0,hr2);
					}
					else {
						t->type = TILE_NE;
						w.p2 = fpos;
						w.p1 = fpos + Vec2D(1,1);
						w.normal = Vec3D(hr2,0,-hr2);
					}
					w.type = WALL_WALL;
					w.sides = WALL_SIDE_FRONT;
					walls.push_back(w);
					t->walls[t->nwalls++] = walls.size()-1;
					break;
				default:
					// do walls
					w.type = WALL_WALL;
					w.sides = WALL_SIDE_FRONT;

					// top
					if (smap[j-1][i]!='.') {
						w.p1 = fpos;
						w.p2 = fpos + Vec2D(1,0);
						w.normal = Vec3D(0,0,1);
						walls.push_back(w);
						t->walls[t->nwalls++] = walls.size()-1;
					}
					// bottom
					if (smap[j+1][i]!='.') {
						w.p2 = fpos + Vec2D(0,1);
						w.p1 = fpos + Vec2D(1,1);
						w.normal = Vec3D(0,0,-1);
						walls.push_back(w);
						t->walls[t->nwalls++] = walls.size()-1;
					}
					// right
					if (smap[j][i+1]!='.') {
						w.p1 = fpos + Vec2D(1,0);
						w.p2 = fpos + Vec2D(1,1);
						w.normal = Vec3D(-1,0,0);
						walls.push_back(w);
						t->walls[t->nwalls++] = walls.size()-1;
					}
					// left
					if (smap[j][i-1]!='.') {
						w.p2 = fpos;
						w.p1 = fpos + Vec2D(0,1);
						w.normal = Vec3D(1,0,0);
						walls.push_back(w);
						t->walls[t->nwalls++] = walls.size()-1;
					}
				}
				t->prepare();
			} // if c
		} // for i
	} // for j
}


void World::draw(Vec2D viewpos, float angle)
{
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glFrontFace(GL_CW); // argh

	Vec3D view(viewpos.x, 0, viewpos.y);
	Vec3D cam = view + Vec3D(cam_dist * cosf(angle*TO_RAD), cam_height, cam_dist * sinf(angle*TO_RAD));
	gluLookAt(cam.x, cam.y, cam.z, view.x, view.y, view.z, 0,1,0);


	Frustum f;
	f.retrieve();


	//const float amb = 3.0f;
	const float amb = 1.0f;
	GLfloat la[4] = {amb, amb, amb, 1.0f};
	GLfloat ld[4] = {0,0,0,1};
	GLfloat lp[4] = {0,-1,0,0};

	glEnable(GL_LIGHTING);

	const float mmb = 0.5f;
	GLfloat lma[4] = {mmb, mmb, mmb, 1.0f};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lma);

	glLightfv(GL_LIGHT0, GL_AMBIENT, la);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, ld);
	glLightfv(GL_LIGHT0, GL_POSITION,lp);

	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glColor4f(1,1,1,1);
	
	glEnable(GL_LIGHT0); // light0 is ambient lighting.

	//glDisable(GL_LIGHTING);


	for (int l=GL_LIGHT1; l<=GL_LIGHT7; l++) {
		glLightfv(l, GL_AMBIENT, ld);

		
		/*
		glLightf(l, GL_CONSTANT_ATTENUATION, 0);
		glLightf(l, GL_LINEAR_ATTENUATION, 0.8f);
		glLightf(l, GL_QUADRATIC_ATTENUATION, 0.5f);
		*/
	
		glLightf(l, GL_CONSTANT_ATTENUATION, 0);
		glLightf(l, GL_LINEAR_ATTENUATION, 0.8f);
		glLightf(l, GL_QUADRATIC_ATTENUATION, 0.1f);


		glDisable(l);
	}

	for (std::map<int,Light>::iterator it = lights.begin(); it != lights.end(); ++it) {
		it->second.prepare();
	}

	glEnable(GL_TEXTURE_2D);

	for (int j=0; j<height; j++) {
		for (int i=0; i<width; i++) {
			Tile *t = getTile(i,j);
			if (t->visible) t->inview = f.intersects(t->c1, t->c2);
			else t->inview = false;
		}
	}

	// cheap trick to avoid sorting by texture - zomg I am lazy
	for (int j=0; j<height; j++) {
		for (int i=0; i<width; i++) {
			Tile *t = getTile(i,j);
			if (t->inview) t->draw();
		}
	}
	/*
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glColor4f(1,0,0,1);
	*/
	for (int j=0; j<height; j++) {
		for (int i=0; i<width; i++) {
			Tile *t = getTile(i,j);
			if (t->inview) t->drawWalls();
		}
	}

	glFrontFace(GL_CCW); // double argh

	for (vector<Object*>::iterator it = objects.begin(); it != objects.end(); ++it) {
        (*it)->draw();
	}

	player.draw();

	for (vector<Treasure>::iterator it = treasures.begin(); it != treasures.end(); ++it) {
		it->draw();
	}
}


void Object::update(float dt)
{
}

void Guard::update(float dt)
{
	// ...
	if (state==0) {
		if (dleft<=0) {

			switch (patcmd) {
			case 'F':
			case 'D':
				pos = tp;
				break;
			case '<':
			case '>':
				dir = td;
				break;
			}

			// new cmd
			patcmd = pattern[patpos++];
			patpos %= pattern.length();

			//fprintf(stderr, "%c\n", patcmd);

			switch (patcmd) {
			case 'F':
				dleft = 1.0f;
				dd = 0.5f;
				tp = pos + Vec2D(cosf(dir*TO_RAD), sinf(dir*TO_RAD));
				break;
			case 'D':
				dleft = hr2*2.0f;
				dd = 0.5f;
				tp = pos + Vec2D(cosf(dir*TO_RAD), sinf(dir*TO_RAD)) * dleft;
				break;
			case '<':
				dleft = 45.0f;
				dd = 45.0f;
				td = dir - 45.0f;
				break;
			case '>':
				dleft = 45.0f;
				dd = 45.0f;
				td = dir + 45.0f;
				break;
			}
		}

		dt *= spd;

		float ddl = dd * dt;
//		if (dleft < ddl) dt = dleft / ddl;
		dleft -= ddl;

		switch (patcmd) {
		case 'F':
		case 'D':
			pos += Vec2D(cosf(dir*TO_RAD), sinf(dir*TO_RAD)) * dt * dd;
			walkcycle += dt * 6.0f;
			break;
		case '<':
			dir -= dd * dt;
			break;
		case '>':
			dir += dd * dt;
			break;
		}
	}

	if (light != -1) {
		Light &l = w->lights[light];
		l.pos = pos;
		l.dir = dir;
	}
}


void Guard::draw()
{
	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, cop->texture);
	glTranslatef(pos.x,rad,pos.y);

	walkscale();

	glRotatef(90-dir, 0, 1, 0);
	glScalef(rad*2.0f,rad*2.0f,rad*2.0f);
    cop->draw();
	glPopMatrix();

}

void Object::walkscale()
{
	float h = 0.1f * sinf(walkcycle);
	glScalef(1.0f+h, 1.0f-h, 1.0f+h);
}



const float DIVLEN = 1.0f / SUBDIV;
const Vec2D pd(DIVLEN*0.5f, DIVLEN*0.5f);
const Vec2D po(DIVLEN, DIVLEN);

Vec3D colValues[SUBDIV * SUBDIV];

void Tile::prepare()
{
	for (int i=0; i<nwalls; i++) w->walls[walls[i]].prepare();
}


void Tile::draw()
{
	glBindTexture(GL_TEXTURE_2D, tex);

	int l = GL_LIGHT1;

	nl = 0;

	for (std::map<int,Light>::iterator it = w->lights.begin(); it != w->lights.end(); ++it) {
		if ((it->second.pos - mpos).lengthSquared() < (it->second.dist*it->second.dist)+4.0f) {
			potLights[nl++] = &it->second;
			// set up light l
			it->second.setup(l++);
		}
		if (nl==7) break;
	}
	for (; l<=GL_LIGHT7; l++) glDisable(l);

	glNormal3f(0,1,0);


	// draw tile
	for (int j=0; j<SUBDIV; j++) {
		for (int i=0; i<SUBDIV; i++) {
			int border=0;

			if (type != TILE_REGULAR) {
				if (type==TILE_NW) {
					if (j==(SUBDIV-i-1)) border = 1;
					if (j>SUBDIV-i-1) continue;
				}
				if (type==TILE_SE) {
					if (j==(SUBDIV-i-1)) border = 2;
					if (j<SUBDIV-i-1) continue;
				}
				if (type==TILE_NE) {
					if (j==i) border = 3;
					if (j>i) continue;
				}
				if (type==TILE_SW) {
					if (j==i) border = 4;
					if (j<i) continue;
				}
				//if ((type==TILE_NE) && (j>i)) continue;
				//if ((type==TILE_SW) && (j<i)) continue;
			}

			Vec2D t0 = Vec2D(DIVLEN * i, DIVLEN * j);
			Vec2D t1 = t0 + po;

			Vec2D p0 = this->pos + t0;
			Vec2D pt = p0 + pd;
			Vec2D p1 = pt + pd;

			// lights for this one
			for (int l=0, ll=GL_LIGHT1; l<nl; l++, ll++) {
				if (potLights[l]->visible(pt)) {
					//if (potLights[l]->visiblePlayer(pt)) glColor3f(0,1,0);
					//else glColor3f(1,0,0);
					glEnable(ll);
				}
				else {
					//glColor3f(0,0,1);
					glDisable(ll);
				}
			}

			//glCallList(lists[j*SUBDIV+i]);

			//glBegin(GL_QUADS);

			if (type == TILE_REGULAR || border==0) {
				glBegin(GL_TRIANGLE_FAN);

				glTexCoord2f(t0.x + DIVLEN * 0.5f, t0.y + DIVLEN * 0.5f);
				glVertex3f(pt.x, 0, pt.y);

				glTexCoord2f(t0.x, t0.y);
				glVertex3f(p0.x, 0, p0.y);

				glTexCoord2f(t1.x, t0.y);
				glVertex3f(p1.x, 0, p0.y);

				glTexCoord2f(t1.x, t1.y);
				glVertex3f(p1.x, 0, p1.y);

				glTexCoord2f(t0.x, t1.y);
				glVertex3f(p0.x, 0, p1.y);

				glTexCoord2f(t0.x, t0.y);
				glVertex3f(p0.x, 0, p0.y);

				glEnd();
			}
			else if (border==1) {
				// NW
				glBegin(GL_TRIANGLES);
				glTexCoord2f(t0.x,t0.y);
				glVertex3f(p0.x, 0, p0.y);

				glTexCoord2f(t1.x, t0.y);
				glVertex3f(p1.x, 0, p0.y);

				glTexCoord2f(t0.x, t1.y);
				glVertex3f(p0.x, 0, p1.y);
				glEnd();
			}
			else if (border==2) {
				// SE
				glBegin(GL_TRIANGLES);

				glTexCoord2f(t0.x, t1.y);
				glVertex3f(p0.x, 0, p1.y);

				glTexCoord2f(t1.x, t0.y);
				glVertex3f(p1.x, 0, p0.y);

				glTexCoord2f(t1.x, t1.y);
				glVertex3f(p1.x, 0, p1.y);

				glEnd();
			}
			else if (border==3) {
				// NE
				glBegin(GL_TRIANGLES);
				glTexCoord2f(t0.x,t0.y);
				glVertex3f(p0.x, 0, p0.y);

				glTexCoord2f(t1.x, t0.y);
				glVertex3f(p1.x, 0, p0.y);

				glTexCoord2f(t1.x, t1.y);
				glVertex3f(p1.x, 0, p1.y);
				glEnd();
			}
			else if (border==4) {
				// SW
				glBegin(GL_TRIANGLES);

				glTexCoord2f(t0.x, t1.y);
				glVertex3f(p0.x, 0, p1.y);

				glTexCoord2f(t0.x, t0.y);
				glVertex3f(p0.x, 0, p0.y);

				glTexCoord2f(t1.x, t1.y);
				glVertex3f(p1.x, 0, p1.y);

				glEnd();
			}


			//glColor3f(1,1,1);

		}
	}

	if (hole) {
		glBindTexture(GL_TEXTURE_2D, holetex);
		glEnable(GL_BLEND);
		glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

		glBegin(GL_QUADS);

		const float y = 0.01f;

		glTexCoord2f(0,0);
		glVertex3f(c1.x,y,c1.z);

		glTexCoord2f(1,0);
		glVertex3f(c2.x,y,c1.z);

		glTexCoord2f(1,1);
		glVertex3f(c2.x,y,c2.z);

		glTexCoord2f(0,1);
		glVertex3f(c1.x,y,c2.z);

		glEnd();

		glDisable(GL_BLEND);
	}

}

void Tile::drawWalls()
{
	for (int l=0, ll=GL_LIGHT1; l<nl; l++, ll++) {
		potLights[l]->setup(ll);
		glEnable(ll);
	}

	// draw walls
	for (int i=0; i<nwalls; i++) {
		Wall *ww = &(w->walls[walls[i]]);
		glBindTexture(GL_TEXTURE_2D, ww->tex);

		glNormal3fv(ww->normal);

		for (int i=0; i<SUBDIV; i++) {
			Vec3D pp0 = ww->bp1 + (ww->dp * (float)i);
			Vec3D pp1 = pp0 + ww->dp;

			/*
			Vec2D ppt(pp0.x, pp0.z);
			
			for (int l=0, ll=GL_LIGHT1; l<nl; l++, ll++) {
				if (potLights[l]->visible(ppt)) {
					glEnable(ll);
				}
				else {
					glDisable(ll);
				}
			}
			*/

			for (int j=0; j<SUBDIV; j++) {
				Vec2D t0 = Vec2D(DIVLEN * i, DIVLEN * j);
				Vec2D t1 = t0 + po;

				const float wh = WALL_HEIGHT;
				
				float y0 = wh - j * wh * DIVLEN;
				float y1 = y0 - wh * DIVLEN;

				glBegin(GL_QUADS);

				glTexCoord2f(t0.x, t0.y);
				glVertex3f(pp0.x, y0, pp0.z);

				glTexCoord2f(t1.x, t0.y);
				glVertex3f(pp1.x, y0, pp1.z);
				
				glTexCoord2f(t1.x, t1.y);
				glVertex3f(pp1.x, y1, pp1.z);

				glTexCoord2f(t0.x, t1.y);
				glVertex3f(pp0.x, y1, pp0.z);

				glEnd();
			}
		}
		
	}
}



bool Light::visible(Vec2D point)
{
	float angle = atan2(point.y-pos.y, point.x-pos.x) / TO_RAD;
	float dist2 = (point-pos).lengthSquared();
	float a2 = angle-dir;
	while (a2<-180.0f) a2+=360.0f;
	while (a2>=180.0f) a2-=360.0f;

	return (dist2 < (dist*dist)) && (fabs(a2) < conesize);

	// shadows? :(
}

bool Light::visiblePlayer(Vec2D point, float dp)
{
	float angle = atan2(point.y-pos.y, point.x-pos.x) / TO_RAD;
	float dist2 = (point-pos).lengthSquared();
	float a2 = angle-dir;
	while (a2<-180.0f) a2+=360.0f;
	while (a2>=180.0f) a2-=360.0f;

	float sdist = dist * dp;
	float scone = conesize * 0.9f;

	// wall occlusion..?

	return (dist2 < (sdist*sdist)) && (fabs(a2) < scone);
}




void Light::setup(GLuint light)
{
	glLightfv(light, GL_POSITION, pos3d);
	glLightfv(light, GL_DIFFUSE, col4);
	glLightfv(light, GL_SPOT_DIRECTION, dir3d);
	glLightf(light, GL_SPOT_CUTOFF, conesize);
	//glLightf(light, GL_SPOT_EXPONENT, 20.0f);
	glLightf(light, GL_SPOT_EXPONENT, 30.0f);
}

void Light::prepare()
{
	pos3d[0] = pos.x;
	pos3d[1] = 0.2f;
	pos3d[2] = pos.y;
	pos3d[3] = 1;

	dir3d[0] = cosf(dir*TO_RAD);
	dir3d[1] = -0.1f;
	dir3d[2] = sinf(dir*TO_RAD);
	dir3d[3] = 0;

	//const float mul = 5.0f;
	const float mul = 3.0f;

	col4[0] = color.x * mul;
	col4[1] = color.y * mul;
	col4[2] = color.z * mul;
	col4[3] = 1;

	while (dir>180.0f) dir -= 360.0f;

}



void Wall::prepare()
{
	bp1 = Vec3D(p1.x, 0, p1.y);
	bp2 = Vec3D(p2.x, 0, p2.y);
	dp = (bp2-bp1) * (1.0f/SUBDIV);
}



void World::update(float dt)
{
	if (victory || gameover) return;

	for (vector<Object*>::iterator it = objects.begin(); it != objects.end(); ++it) {
        (*it)->update(dt);
	}
	
	player.update(dt);

	// check player visibility

	bool a = false;
	for (map<int,Light>::iterator it = lights.begin(); it != lights.end(); ++it) {
		Light &l = it->second;
		if (l.visiblePlayer(player.pos, 0.4f)) {
			// game over
			awareness = 1.0f;
			a = true;
		}
		else if (l.visiblePlayer(player.pos, 0.6f)) {
			awareness += dt * 0.4f;
			if (awareness > 1.0f) awareness = 1.0f;
			a = true;
		}
		else if (l.visiblePlayer(player.pos, 0.8f)) {
			awareness += dt * 0.2f;
			if (awareness > 1.0f) awareness = 1.0f;
			a = true;
		}
	}

	for (vector<Object*>::iterator it = objects.begin(); it != objects.end(); ++it) {
		Object *o = *it;
		if (o->type == GUARD) {
			float d = (o->pos-player.pos).length();
			float tr = player.rad + o->rad;
			if (d <= tr) {
				awareness = 1.0f;
				a = true;
			}
			else if (d <= 1.8f*tr) {
				awareness += dt * 0.3f;
				if (awareness > 1.0f) awareness = 1.0f;
				a = true;
			}

		}
	}

    bool canwin = true;

	for (vector<Treasure>::iterator it = treasures.begin(); it != treasures.end(); ++it) {
		if (!it->taken) canwin = false;
		float d = (it->pos-player.pos).length();
		float tr = player.rad + it->rad;
		if (d <= tr && !it->taken) {
			it->taken = true;
			collected++;
		}

	}

	if (canwin && getTile((int)player.pos.x, (int)player.pos.y)->hole) {
        victory = true;
	}


	if (!a && awareness > 0 && awareness < 1) {
		awareness -= dt * 0.05f;
		if (awareness<0) awareness = 0;
	}

	if (awareness >= 1) gameover = true;

}


void World::gameStart()
{
    awareness = 0;
	player.pos = pstart;
	player.dir = 0;
	victory = false;
	gameover = false;
	for (vector<Treasure>::iterator it = treasures.begin(); it != treasures.end(); ++it) {
		it->taken = false;
	}
	for (vector<Object*>::iterator it = objects.begin(); it != objects.end(); ++it) {
        Object *o = *it;
		if (o->type==GUARD) {
			Guard *g = reinterpret_cast<Guard*>(o);
			g->pos = g->pos0;
			g->dir = g->dir0;
			g->patcmd = 0;
			g->patpos = 0;
			g->dleft = 0;
		}
	}
	collected = 0;
}

void Player::draw()
{
	Tile *t = w->getTile((int)pos.x, (int)pos.y);
	
	for (int l=0, ll=GL_LIGHT1; l<t->nl; l++, ll++) {
		t->potLights[l]->setup(ll);
		glEnable(ll);
	}

	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, ninja->texture);
	glTranslatef(pos.x,rad,pos.y);

	walkscale();

	glRotatef(90-dir, 0, 1, 0);
	glScalef(rad*2.0f,rad*2.0f,rad*2.0f);
    ninja->draw();
	glPopMatrix();

}

void Player::update(float dt)
{
	Vec2D vdir(cosf(dir*TO_RAD), sinf(dir*TO_RAD));
	Vec2D vstr(cosf((dir+90.0f)*TO_RAD), sinf((dir+90.0f)*TO_RAD));
	
	if ((fabs(fw)+fabs(strafe)) > 0) {
		walkcycle += dt * 6.0f;
	}

	Vec2D npos = pos + ((vdir * fw)+(vstr * strafe)) * dt * 2.0f;
    dir += turn * dt * 90.0f;

	float rr = rad*rad;

	// coll.det
	bool coll = false;
	int ix = (int)pos.x;
	int iy = (int)pos.y;
	for (int j=iy-1; j<=iy+1; j++) {
		for (int i=ix-1; i<=ix+1; i++) {
			if (i>=0 && j>=0 && i<w->width && j<w->height) {
				Tile *t = w->getTile(i,j);
				for (int k=0; k<t->nwalls; k++) {
					float sign, dist, closest;
					Wall &wall = w->walls[t->walls[k]];
					classifyPoint(npos, wall.p1, wall.p2, dist, sign, closest);
					if ((sign<=0) && (dist<rad)) {
						// collision
						if (closest>=0 && closest<=1) {
							coll = true;
							break;
						}
					}
					if ((wall.p1-npos).lengthSquared() < rr || (wall.p2-npos).lengthSquared() < rr) {
						coll = true;
						break;
					}
				}
			}
			if (coll) break;
		}
		if (coll) break;
	}

	if (!coll) pos = npos;
}

void Treasure::draw()
{
	if (taken) return;

	Tile *t = w->getTile((int)pos.x, (int)pos.y);
	
	for (int l=0, ll=GL_LIGHT1; l<t->nl; l++, ll++) {
		t->potLights[l]->setup(ll);
		glEnable(ll);
	}

	glPushMatrix();
	glBindTexture(GL_TEXTURE_2D, grail->texture);

	glTranslatef(pos.x,rad,pos.y);

	glScalef(rad*2.0f,rad*2.0f,rad*2.0f);
    grail->draw();
	glPopMatrix();


}