#ifndef WORLD_H
#define WORLD_H

#include "vec3d.h"
#include "video.h"
#include <vector>
#include <string>
#include <map>

const float TILE_SIZE = 1.0f;

const float PI = 3.14159265358f;
const float TO_RAD = PI / 180.0f;


struct World;


struct Light {
    Vec2D pos;
	float dir, conesize, dist;
	World *w;
	Vec3D color;

	GLfloat pos3d[4], dir3d[4], col4[4];

	bool visible(Vec2D point);
	bool visiblePlayer(Vec2D point, float dp);
	void setup(GLuint light);
	void prepare();
};


enum WallSides {
	WALL_SIDE_FRONT,
	WALL_SIDE_BOTH
};

enum WallTypes {
	WALL_WALL,
	WALL_DOOR
};

struct Wall {
	int sides, type;
	bool open;

	Vec2D p1, p2;

    Texture tex;

	bool horiz;

	Vec3D bp1, bp2, dp;
	Vec3D normal;

	void prepare();

	bool occludes()
	{
		return (type==WALL_DOOR) ? open : true;
	}
};


enum TileType {
	TILE_NONE,
	TILE_REGULAR,
	TILE_NE,
	TILE_NW,
	TILE_SE,
	TILE_SW
};

//const int SUBDIV = 6;

const int SUBDIV = 8;


struct Tile {
	int type;
	Texture tex;

	Tile(): nwalls(0), w(0), visible(false), nl(0), hole(false)
	{
		walls[0]=walls[1]=walls[2]=walls[3]=0;
	}

	bool visible,inview,hole;
	int nwalls;
	size_t walls[4];
	World *w;

	Vec2D pos, mpos;

	Vec3D c1,c2;

	void draw();
	void drawWalls();
	void prepare();

	//GLuint lists[SUBDIV*SUBDIV];

	Light *potLights[7];
	int nl;
};

enum ObjectType {
	GUARD,
	TREASURE
};


struct Object {

	int type;

	Vec2D pos;
	float dir, rad;
	World *w;

	float walkcycle;

	virtual void update(float dt);
	virtual void draw() {}
	void walkscale();
};

struct Treasure: public Object {
	bool taken;

	Treasure()
	{
		type = TREASURE;
		rad = 0.2f;
		taken = false;
	}

	virtual void draw();
};

struct Player: public Object {
    float fw, turn, strafe;

	Player()
	{
		fw = turn = strafe = 0;
		rad = 0.333f*0.5f;
		walkcycle = 0;
	}

	void update(float dt);
	void draw();
};


struct Guard: public Object {
	int light;
	std::string pattern;
	int patpos;
	float dleft, dd, td, spd;
	char patcmd;
	Vec2D tp;
	Vec2D pos0;
	float dir0;

	int state;

	Guard()
	{
		rad = 0.333f*0.5f;
		light = -1;
		state = 0;
		patcmd = 0;
		patpos = 0;
		dleft = 0;
		walkcycle = 0;
		type = GUARD;
		spd = 1.0f;
	}

	void update(float dt);
	void draw();
};


struct World {
	std::vector<Object*> objects;
	std::map<int,Light> lights;
	std::vector<Wall> walls;

	std::vector<std::string> smap;

	Player player;
	std::vector<Treasure> treasures;

	World() : lid(0), tiles(0)
	{
	}
	~World()
	{
		delete[] tiles;
	}

	int lid;
	int width, height;
	Tile *tiles;

	int collected, total;

	bool victory,gameover;

	Tile *getTile(int x, int y)
	{
		return &tiles[y*width+x];
	}

	void init(const char *filename);
	void draw(Vec2D viewpos, float angle);
	void update(float dt);


	void gameStart();
	Vec2D pstart,tstart;


	float awareness;

};






#endif
